MODULE ExtraRAM;
(* =========================================================================  
   Example Cortex-M3 Oberon Program  
  
   Description:
     Illustrates the use of the additional 16kB blocks of AHB SRAM present on 
     LPC175x/6x MCUs with a total of 32kB or more of SRAM
     
     Run the program twice - once using the default settings and once with 
     the AHB SRAM settings for your MCU and compare the Stack Usage / Heap Usage 
     for each:
     
     1. Default settings: (Shares the CPU RAM between the stack and the heap)
        Heap Start = 010000200H, 
        Heap Limit = 0H 
        
     2. LPC1758/59/63/65/66/67/68/69 (Uses extra 32kB of AHB SRAM for the heap)
        Heap Start = 02007C000H
        Heap Limit = 020084000H 
        
     3. LPC1754/56 (Uses extra 16kB of AHB SRAM for the heap)
        Heap Start = 02007C000H
        Heap Limit = 020080000H 
     
   References: 

     NXP UM10360 LPC17xx User manual
     Oberon for Cortex-M3 Microcontrollers
   
   (c) 2013 CFB Software   
   http://www.astrobe.com  
 
 ========================================================================= *)

IMPORT LinkOptions, Main, MCU, Out, Storage;

TYPE
  DynArray = POINTER TO RECORD 
    val: ARRAY 5000 OF INTEGER 
  END;
  
VAR
  bigArray: DynArray;
  
PROCEDURE OutTitle(title: ARRAY OF CHAR);
CONST
  width = 20;
VAR 
  i, count: INTEGER;
BEGIN
  Out.String(title);
  count := width - STRLEN(title);
  FOR i := 1 TO count DO 
    Out.Char(" ")
  END
END OutTitle;
   
PROCEDURE OutFreq(title: ARRAY OF CHAR; item: INTEGER);
BEGIN
  OutTitle(title);
  Out.Int(item, 0);
  Out.String(" Hz");
  Out.Ln
END OutFreq;

PROCEDURE OutAddr(title: ARRAY OF CHAR; item: INTEGER);
BEGIN
  OutTitle(title);
  Out.Hex(item, 0);
  Out.Ln
END OutAddr;

PROCEDURE OutSize(title: ARRAY OF CHAR; item: INTEGER);
BEGIN
  OutTitle(title);
  Out.Int(item, 0);
  Out.String(" Bytes");
  Out.Ln
END OutSize;

PROCEDURE DisplayStats();
BEGIN
  OutAddr("Heap Start:", Storage.HeapStart());
  OutSize("Heap Available:", Storage.HeapAvailable());
  OutSize("Heap Used:", Storage.HeapUsed());
  OutAddr("Stack Start:", Storage.StackStart());
  OutSize("Stack Available:", Storage.StackAvailable());
  OutSize("Stack Used:", Storage.StackUsed());
  Out.String("---------------");
  Out.Ln()
END DisplayStats;

BEGIN
  (* Check the initial memory usage *)
  DisplayStats();
  (* Allocate memory for the dynamic array and check the memory usage *)
  NEW(bigArray);
  DisplayStats();
  (* Free the memory used by the dynamic array and check the memory usage *)
  DISPOSE(bigArray);
  DisplayStats();
END ExtraRAM.

